import django_tables2 as tables
from django.utils.safestring import mark_safe


row_onclick = "if(window.hasOwnProperty('goTo')){if(typeof window.goTo === 'function'){goTo(this);}}"

class MessageTable(tables.Table):
    read = tables.Column(attrs={'th': {'class': 'read_column',}}, order_by='read')
    short_message = tables.Column(verbose_name="Message", order_by='message')
    created_at = tables.Column(verbose_name="Received At")
    def render_read(self, record):
        html = '<i class="fa fa-envelope"></i>'
        if record.read:
            html = '<i class="fa fa-envelope-o"></i>'
        return mark_safe(html)
    def render_short_message(self, record, value):
        html = record.image_tag() + ' ' + value
        return mark_safe(html)
    class Meta:
        empty_text = "No messages"
        row_attrs = {
            'data-id': lambda record: record.pk,
            'onclick': row_onclick,
            'class': 'table-row',
        }


class NotificationTable(tables.Table):
    read = tables.Column(attrs={'th': {'class': 'read_column',}}, order_by='read')
    short_notification = tables.Column(verbose_name="Message", order_by='notification')
    created_at = tables.Column(verbose_name="Received At")
    def render_read(self, record):
        html = '<i class="fa fa-envelope"></i>'
        if record.read:
            html = '<i class="fa fa-envelope-o"></i>'
        return mark_safe(html)
    def render_short_notification(self, record, value):
        icon = ''
        if record.icon_class:
            icon = '<i class="%s"></i>' % record.icon_class
        html = icon + value
        return mark_safe(html)
    class Meta:
        empty_text = "No notifications"
        row_attrs = {
            'data-id': lambda record: record.pk,
            'onclick': row_onclick,
            'class': 'table-row',
        }



from django.shortcuts import render, redirect, reverse
from django.views.generic import TemplateView, RedirectView, FormView
from django.views.generic.edit import CreateView, UpdateView
from django.contrib.auth.views import LoginView, PasswordResetView, PasswordResetConfirmView, \
    PasswordChangeView
from django.contrib import messages as django_messages
from django.contrib.auth import login
from django_tables2 import RequestConfig

from cosmicdb.forms import CosmicAuthenticationForm, CosmicSignupUserForm, CosmicPasswordResetForm, \
    CosmicSetPasswordForm, CosmicPasswordChangeForm
from cosmicdb.models import User
from cosmicdb.tables import NotificationTable, MessageTable


class CosmicSaveErrorDialogsMixin(object):
    success_message = 'Saved'
    extra_context = {}

    def get_context_data(self, **kwargs):
        context = super(CosmicSaveErrorDialogsMixin, self).get_context_data(**kwargs)
        context['extra_context'] = self.extra_context
        return context

    def get_success_message(self, cleaned_data):
        return self.success_message % cleaned_data

    def form_valid(self, form):
        self.extra_context['page_error'] = False
        response = super(CosmicSaveErrorDialogsMixin, self).form_valid(form)
        success_message = self.get_success_message(form.cleaned_data)
        if success_message:
            django_messages.success(self.request, success_message)
        return response

    def form_invalid(self, form):
        self.extra_context['page_error'] = True
        return super(CosmicSaveErrorDialogsMixin, self).form_invalid(form)


class CosmicLoginView(LoginView):
    form_class = CosmicAuthenticationForm

    def get_success_url(self):
        return reverse('dashboard')


class CosmicSignupView(FormView):
    form_class = CosmicSignupUserForm
    template_name = 'cosmicdb/signup.html'

    def get_success_url(self):
        return reverse('dashboard')

    def form_valid(self, form):
        new_user = User.objects.create_user(**form.cleaned_data)
        login(self.request, new_user)
        return super(CosmicSignupView, self).form_valid(form)


class CosmicPasswordResetView(PasswordResetView):
    form_class = CosmicPasswordResetForm


class CosmicPasswordResetConfirmView(PasswordResetConfirmView):
    form_class = CosmicSetPasswordForm


class CosmicPasswordChangeView(PasswordChangeView):
    form_class = CosmicPasswordChangeForm


class Home(RedirectView):
    def get_redirect_url(self, *args, **kwargs):
        return reverse('dashboard')


class Dashboard(TemplateView):
    template_name = 'cosmicdb/dashboard.html'


def notifications(request, id = None):
    if not request.user.is_authenticated:
        return redirect(reverse('home'))
    if id is None:
        notifications = request.user.usersystemnotification_set.order_by('-created_at')
        table = NotificationTable(notifications)
        RequestConfig(request, paginate={'per_page': 10}).configure(table)
        return render(request, 'cosmicdb/notifications_all.html', {'table': table})
    else:
        notification = request.user.usersystemnotification_set.get(id=id)
        notification.read = True
        notification.save()
        return render(request, 'cosmicdb/notifications_single.html', {'notification': notification})


def messages(request, id=None):
    if not request.user.is_authenticated:
        return redirect(reverse('home'))
    if id is None:
        messages = request.user.usersystemmessage_set.order_by('-created_at')
        table = MessageTable(messages)
        RequestConfig(request, paginate={'per_page': 10}).configure(table)
        return render(request, 'cosmicdb/messages_all.html', {'table': table})
    else:
        message = request.user.usersystemmessage_set.get(id=id)
        message.read = True
        message.save()
        return render(request, 'cosmicdb/messages_single.html', {'message': message})



import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = '7nti!w-x$i=twbb=rwqg-u@un(!&^jzt7dhu(q-pnk2f4h2sau'

DEBUG = True

ALLOWED_HOSTS = [
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

INSTALLED_APPS = [
    'cosmicdb',
    'dal',
    'dal_select2',
    'crispy_forms',
    'django_tables2',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

ROOT_URLCONF = 'cosmicdb.urls'

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

STATIC_URL = '/static/'

COSMICDB_SITE_TITLE = 'CosmicDB'
COSMICDB_ALLOW_SIGNUP = False
AUTH_USER_MODEL = 'cosmicdb.User'
LOGIN_URL = '/login/'
CRISPY_TEMPLATE_PACK = 'bootstrap3'
DJANGO_TABLES2_TEMPLATE = 'django_tables2/bootstrap-responsive.html'
EMAIL_USE_TLS = True
EMAIL_HOST = 'smtp.gmail.com'
EMAIL_PORT = 587
EMAIL_HOST_USER = 'test@test.com'
EMAIL_HOST_PASSWORD = 'testpassword'
DEFAULT_FROM_EMAIL = EMAIL_HOST_USER
DEFAULT_FROM_EMAIL_NAME = COSMICDB_SITE_TITLE



from django import forms
from django.contrib.auth.forms import AuthenticationForm, PasswordResetForm, SetPasswordForm, \
    PasswordChangeForm
from django.contrib.auth import password_validation
from django.utils.html import format_html
from crispy_forms.helper import FormHelper, Layout
from crispy_forms.layout import HTML, Div, Submit, BaseInput, Hidden, Field

from cosmicdb.models import User

class CosmicFormHelper(FormHelper):
    def __init__(
        self,
        form,
        no_submit_row=False,
        submit_row=None,
        submit_value=None,
        submit_classes=None,
        submit_div_classes=None,
        field_layout=(),
        *args,
        **kwargs
    ):
        super(CosmicFormHelper, self).__init__(*args, **kwargs)
        field_layout = field_layout
        if not field_layout:
            field_layout = tuple(field.name for field in form.visible_fields()) + tuple(field.name for field in form.hidden_fields())
        if submit_row is None and not no_submit_row:
            submit_value = submit_value
            if submit_value is None:
                submit_value = 'Save'
            submit_classes = submit_classes
            if submit_classes is None:
                submit_classes = ''
            submit_div_classes = submit_div_classes
            if submit_div_classes is None:
                submit_div_classes = ''
            submit_row = Div(Submit('save', submit_value, type='submit', css_class=submit_classes), css_class=submit_div_classes)
        if no_submit_row or submit_row is None:
            submit_row = Div()
        field_layout = Layout(
            *field_layout,
            submit_row,
        )
        self.layout = field_layout


class CosmicFormsetHelper(FormHelper):
    template = 'bootstrap3/table_inline_formset.html'

    def __init__(self, *args, **kwargs):
        super(CosmicFormsetHelper, self).__init__(*args, **kwargs)
        self.form_tag = False


class CosmicAuthenticationForm(AuthenticationForm):
    def __init__(self, *args, **kwargs):
        super(CosmicAuthenticationForm, self).__init__(*args, **kwargs)
        self.helper = CosmicFormHelper(self, submit_value='Sign In', submit_classes='btn-block btn-flat')


class CosmicSignupUserForm(forms.ModelForm):
    def clean_email(self):
        email = self.cleaned_data['email']
        if email == '' or email is None:
            raise forms.ValidationError('You must fill in your email address.')
        if User.objects.filter(email=email).count() > 0:
            raise forms.ValidationError('This email is in use.')
        return email
    def clean_username(self):
        username = self.cleaned_data['username']
        if len(username) < 5:
            raise forms.ValidationError('Username must be 5 or more characters.')
        return username
    def clean_password(self):
        password = self.cleaned_data.get('password')
        self.instance.username = self.cleaned_data.get('username')
        password_validation.validate_password(password, self.instance)
        return password

    def __init__(self, *args, **kwargs):
        super(CosmicSignupUserForm, self).__init__(*args, **kwargs)
        self.fields['password'].widget = forms.PasswordInput()
        self.helper = CosmicFormHelper(self, submit_value='Create account', submit_div_classes='box-footer')

    class Meta:
        model = User
        fields = ('username', 'email', 'password')
        help_items = [format_html('<li>{}</li>', help_text) for help_text in password_validation.password_validators_help_texts()]
        help_text_html = '<ul class="help-block">%s</ul>' % ''.join(help_items)
        help_texts = {
            'password': help_text_html,
        }


class CosmicPasswordResetForm(PasswordResetForm):
    def __init__(self, *args, **kwargs):
        super(CosmicPasswordResetForm, self).__init__(*args, **kwargs)
        self.helper = CosmicFormHelper(self, submit_value='Send Reset Email', submit_classes='btn-block btn-flat')


class CosmicSetPasswordForm(SetPasswordForm):
    def __init__(self, *args, **kwargs):
        super(CosmicSetPasswordForm, self).__init__(*args, **kwargs)
        self.helper = CosmicFormHelper(self, submit_value='Reset Password', submit_classes='btn-block btn-flat')


class CosmicPasswordChangeForm(PasswordChangeForm):
    def __init__(self, *args, **kwargs):
        super(CosmicPasswordChangeForm, self).__init__(*args, **kwargs)
        self.helper = CosmicFormHelper(self, submit_value='Change password', submit_div_classes='box-footer')
        self.helper.form_class = 'form-horizontal'
        self.helper.label_class = 'col-sm-4'
        self.helper.field_class = 'col-sm-6'



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin

from cosmicdb.models import User, UserSystemMessage, UserSystemNotification


class UserSystemMessageInlineAdmin(admin.TabularInline):
    model = UserSystemMessage


class UserSystemNotificationInlineAdmin(admin.TabularInline):
    model = UserSystemNotification


class UserProfileAdmin(UserAdmin):
    inlines = [
        UserSystemMessageInlineAdmin,
        UserSystemNotificationInlineAdmin,
    ]


admin.site.register(User, UserProfileAdmin)



from django.conf import settings
from django import template
from django.urls import reverse
from django.utils.safestring import mark_safe

register = template.Library()


@register.simple_tag
def get_site_title():
    return settings.COSMICDB_SITE_TITLE or "CosmicDB"

@register.simple_tag
def get_signup_link_login():
    if hasattr(settings, 'COSMICDB_ALLOW_SIGNUP'):
        if settings.COSMICDB_ALLOW_SIGNUP or False:
            return mark_safe('<a href="%s">Sign up</a>' % reverse('signup'))
    return ''




from django.contrib import admin
from django.conf import settings
from django.views.generic.base import RedirectView
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.conf.urls import url
from django.contrib.auth.decorators import login_required
from django.contrib.auth.views import logout, \
    password_change_done, password_reset_done, password_reset_complete

from cosmicdb.views import CosmicLoginView, CosmicSignupView, CosmicPasswordResetView, \
    CosmicPasswordResetConfirmView, CosmicPasswordChangeView, \
    Home, Dashboard, notifications, messages

favicon_view = RedirectView.as_view(url=static('cosmicdb/img/favicon.ico'), permanent=True)

urlpatterns = [
    url(r'^admin/', admin.site.urls),
    url(r'^login/$', CosmicLoginView.as_view(), name='login'),
    url(r'^logout/$', logout, name='logout'),
    url(r'^password_change/$', CosmicPasswordChangeView.as_view(), name='password_change'),
    url(r'^password_change_done/$', password_change_done, name='password_change_done'),
    url(r'^password_reset/$', CosmicPasswordResetView.as_view(), name='password_reset'),
    url(r'^password_reset_done/$', password_reset_done, name='password_reset_done'),
    url(r'^password_reset_confirm/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',CosmicPasswordResetConfirmView.as_view(), name='password_reset_confirm'),
    url(r'^password_reset_complete/$', password_reset_complete, name='password_reset_complete'),
    url(r'^favicon\.ico$', favicon_view, name='favicon_default'),
    url(r'^$', Home.as_view(), name='home'),
    url(r'^dashboard/$', login_required(Dashboard.as_view()), name='dashboard'),
    url(r'^notifications/$', notifications, name='notifications'),
    url(r'^notifications/(?P<id>[0-9]+)/$', notifications, name='notifications'),
    url(r'^messages/$', messages, name='messages'),
    url(r'^messages/(?P<id>[0-9]+)/$', messages, name='messages'),
]

if hasattr(settings, 'COSMICDB_ALLOW_SIGNUP'):
    if settings.COSMICDB_ALLOW_SIGNUP:
        urlpatterns = urlpatterns + [
            url(r'^signup/$', CosmicSignupView.as_view(), name='signup'),
        ]



from django.db import models
from django.contrib.auth.models import AbstractUser
from django.utils.safestring import mark_safe
from bs4 import BeautifulSoup

class User(AbstractUser):

    def unread_notification_no(self):
        return self.usersystemnotification_set.filter(read=False).count()

    def unread_notifications(self):
        return self.usersystemnotification_set.filter(read=False).order_by('-created_at')[:3]

    def read_notifications(self):
        return self.usersystemnotification_set.filter(read=True).order_by('-created_at')[:3]

    def read_notification_no(self):
        return self.usersystemnotification_set.filter(read=True).count()

    def unread_message_no(self):
        return self.usersystemmessage_set.filter(read=False).count()

    def unread_messages(self):
        return self.usersystemmessage_set.filter(read=False).order_by('-created_at')[:3]

    def read_messages(self):
        return self.usersystemmessage_set.filter(read=True).order_by('-created_at')[:3]

    def read_message_no(self):
        return self.usersystemmessage_set.filter(read=True).count()

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return u'%s' % (self.username)

class UserSystemMessage(models.Model):
    user = models.ForeignKey('User', on_delete=models.PROTECT)
    subject = models.CharField(max_length=30)
    message = models.TextField()
    def short_message(self):
        msg_no_html = BeautifulSoup(self.message, "html.parser").get_text()
        return msg_no_html[:30]+'..'
    image_path = models.CharField(max_length=100, blank=True, default='')
    def image_tag(self):
        if self.image_path != '':
            url = mark_safe('<img src="%s" height="40" />' % (self.image_path))
        else:
            url = ''
        return url
    read = models.BooleanField(default=False)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return u'%s - %s %s' % (self.user, self.subject, self.short_message())

class UserSystemNotification(models.Model):
    user = models.ForeignKey('User', on_delete=models.PROTECT)
    notification = models.TextField()
    def short_notification(self):
        notification_no_html = BeautifulSoup(self.notification, "html.parser").get_text()
        return notification_no_html[:30]+'..'
    icon_class = models.CharField(max_length=50, blank=True)
    read = models.BooleanField(default=False)

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    def __str__(self):
        return u'%s - %s' % (self.user, self.short_notification())
